/**
 * gclc:fr.bigeon.gclc.command.ScriptExecution.java
 * Created on: Jun 12, 2016
 */
package fr.bigeon.gclc.command;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.List;

import fr.bigeon.gclc.ConsoleApplication;
import fr.bigeon.gclc.GCLCConstants;
import fr.bigeon.gclc.exception.CommandParsingException;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.CommandRunExceptionType;

/** <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class ScriptExecution extends Command {

    private final ConsoleApplication application;
    private final String commentPrefix;

    /** @param name
     * @param application
     * @param commentPrefix */
    public ScriptExecution(String name, ConsoleApplication application,
            String commentPrefix) {
        super(name);
        this.application = application;
        this.commentPrefix = commentPrefix;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#execute(java.lang.String[]) */
    @Override
    public void execute(String... args) throws CommandRunException {
        if (args.length == 0) {
            throw new CommandRunException(CommandRunExceptionType.USAGE,
                    "Expecting a file", this);
        }
        List<String> argsList = Arrays.asList(args);
        String scriptFile = argsList.remove(0);
        Object[] params = argsList.toArray();
        try (FileReader fReader = new FileReader(new File(scriptFile));
             BufferedReader reader = new BufferedReader(fReader)) {
            String cmd;
            for (int i = 1; i < args.length; i++) {
                params[i - 1] = args[i];
            }
            while ((cmd = reader.readLine()) != null) {
                if (cmd.startsWith(" ") || cmd.startsWith("\t")) {
                    throw new CommandRunException(
                            "Invalid command in script (line starts with space character)",
                            this);
                }
                if (cmd.isEmpty() || cmd.startsWith(commentPrefix)) {
                    continue;
                }
                String cmdLine = MessageFormat.format(cmd, params);
                List<String> ps = GCLCConstants.splitCommand(cmdLine);
                String command = ps.remove(0);
                application.executeSub(command, ps.toArray(new String[0]));
            }
        } catch (CommandParsingException e) {
            throw new CommandRunException("Invalid command in script (" +
                                          e.getLocalizedMessage() + ")",
                    e, this);
        } catch (IOException e) {
            throw new CommandRunException(
                    "Unable to read script (" + e.getLocalizedMessage() + ")",
                    e, this);
        }
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usageDetail() */
    @Override
    protected String usageDetail() {
        StringBuilder builder = new StringBuilder();
        builder.append(
                "  scriptfile: path to the file containing the script to execute.");
        builder.append(System.lineSeparator());
        builder.append(System.lineSeparator());
        builder.append(
                "  The script file must contain one line commands. The lines must never");
        builder.append(System.lineSeparator());
        builder.append(
                "start with whitespace characters. The lines starting with");
        builder.append(System.lineSeparator());
        builder.append("\"" + commentPrefix +
                       "\" will be ignored as well as empty lines.");
        builder.append(System.lineSeparator());

        return builder.toString();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#tip() */
    @Override
    public String tip() {
        return "Execute a script";
    }

}
